/**
 * Component description
 * @moduleName BytedanceAds
 * @snippet BytedanceAds.snippet
 */
bento.define('modules/bytedanceads', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    // ------ FUNCTIONS ------
    var nothing = function () {
        // do nothing
    };
    var error = function (err) {
        if (!err) {
            return;
        }
        console.log('ByteDance Ads Error: ', err);
    };
    var prepareFullscreenVideoLoop = function () {
        var tween = new Entity({
            name: 'tween',
            global: true,
            components: [
                new Tween.TweenBehavior({ in: 600,
                    onComplete: function () {
                        BytedanceModule.prepareFullscreenVideo(nothing, prepareFullscreenVideoLoop);
                    }
                })
            ]
        });
        Bento.objects.attach(tween);
    };
    var prepareRewardedLoop = function () {
        var tween = new Entity({
            name: 'tween',
            global: true,
            components: [
                new Tween.TweenBehavior({ in: 600,
                    onComplete: function () {
                        BytedanceModule.prepareRewarded(nothing, prepareRewardedLoop);
                    }
                })
            ]
        });
        Bento.objects.attach(tween);
    };

    // --- VARS ---
    var hasFullscreenVideoAd = false;
    var isProcessingFullscreenVideo = false;
    var fullscreenVideoCodeID;
    var autoLoadFullscreenVideoAds = true;
    var fullscreenVideoLoadCallback;
    var fullscreenVideoLoadFailedCallback;
    var fullscreenVideoCompleteCallback;

    var hasRewardedAd = false;
    var isProcessingRewarded = false;
    var rewardedCodeID;
    var autoLoadRewardedAds = true;
    var rewardedLoadCallback;
    var rewardedLoadFailedCallback;
    var rewardedCompleteCallback;

    // --- MODULE ---
    var BytedanceModule = {
        initialize: nothing,
        prepareFullscreenVideo: nothing,
        canShowFullscreenVideo: function () {
            return Utils.isDev();
        },
        showFullscreenVideo: nothing,
        prepareRewarded: nothing,
        canShowRewarded: function () {
            return Utils.isDev();
        },
        showRewarded: nothing
    };
    // Check if we can access iron source, if we can, get it ready
    if (window.cordova && window.cordova.plugins && window.cordova.plugins.toutiao) {
        var BytedancePlugin = window.cordova.plugins.toutiao;

        BytedanceModule = {
            initialize: function (options, success, failure) {
                if (Utils.isDefined(options.autoLoadInteractionAds)) {
                    autoLoadFullscreenVideoAds = options.autoLoadFullscreenVideoAds;
                }
                if (Utils.isDefined(options.autoLoadRewardedAds)) {
                    autoLoadRewardedAds = options.autoLoadRewardedAds;
                }
                if (!options.appID) {
                    Utils.log('Bytedance Ads Initialization failed!', 'No App ID!');
                    return;
                }
                if (!options.appName) {
                    Utils.log('Bytedance Ads Initialization failed!', 'No App Name!');
                    return;
                }
                fullscreenVideoCodeID = options.fullscreenVideoCodeID;
                rewardedCodeID = options.rewardedCodeID;
                BytedancePlugin.initSDK({
                    appID: options.appID,
                    appName: options.appName,
                    textureView: true,
                    showNotificationBar: true,
                    debug: true,
                    showPageOnScreenLock: true,
                    titleBarTheme: 1
                }, function () {
                    console.log('Bytedance Ads Initialized!');
                    if (success) {
                        success();
                    }
                    if (autoLoadFullscreenVideoAds) {
                        prepareFullscreenVideoLoop();
                    }
                    if (autoLoadRewardedAds) {
                        prepareRewardedLoop();
                    }
                }, function (e) {
                    Utils.log('Bytedance Ads Initialization failed! Error: ', e);
                    if (failure) {
                        failure(e);
                    }
                });
            },
            prepareFullscreenVideo: function (success, failure) {
                var screenSize = Utils.getScreenSize();
                var AdSize = new Vector2(screenSize.width, screenSize.height);

                fullscreenVideoLoadCallback = success;
                fullscreenVideoLoadFailedCallback = failure;
                BytedancePlugin.loadFullscreenVideoAd({
                    codeID: fullscreenVideoCodeID,
                    imageWidth: AdSize.x,
                    imageHeight: AdSize.y,
                    autoShow: true
                }, nothing, error);
            },
            canShowFullscreenVideo: function (yes, no) {
                if (hasFullscreenVideoAd && !isProcessingRewarded) {
                    if (yes) {
                        yes();
                    }
                } else {
                    if (no) {
                        no();
                    }
                }
            },
            showFullscreenVideo: function (onComplete, failure) {
                var onFailure = function (e) {
                    isProcessingFullscreenVideo = false;
                    if (failure) {
                        failure(e);
                    } else {
                        error(e);
                    }
                };

                if (hasFullscreenVideoAd && !isProcessingFullscreenVideo) {
                    isProcessingFullscreenVideo = true;
                    fullscreenVideoCompleteCallback = onComplete;
                    BytedancePlugin.showFullscreenVideoAd(nothing, onFailure);
                } else {
                    onFailure("No ad, or already watching an ad.");
                }
            },
            prepareRewarded: function (success, failure) {
                var screenSize = Utils.getScreenSize();
                var AdSize = new Vector2(screenSize.width, screenSize.height);

                rewardedLoadCallback = success;
                rewardedLoadFailedCallback = failure;
                BytedancePlugin.loadRewardVideoAd({
                    codeID: rewardedCodeID,
                    imageWidth: AdSize.x,
                    imageHeight: AdSize.y,
                    rewardName: '',
                    amount: 0,
                    autoShow: false
                }, nothing, error);
            },
            canShowRewarded: function () {
                return hasRewardedAd && !isProcessingRewarded;
            },
            showRewarded: function (onComplete, failure) {
                var onFailure = function (e) {
                    isProcessingRewarded = false;
                    if (failure) {
                        failure(e);
                    } else {
                        error(e);
                    }
                };

                if (hasRewardedAd && !isProcessingRewarded) {
                    isProcessingRewarded = true;
                    rewardedCompleteCallback = onComplete;
                    BytedancePlugin.showRewardVideoAd(nothing, onFailure);
                }
            },
        };
    }

    //interaction callbacks
    document.addEventListener('fullscreenVideoAdVideoDataDidLoad', function (data) {
        hasFullscreenVideoAd = true;
        if (fullscreenVideoLoadCallback) {
            fullscreenVideoLoadCallback(data);
        }
    }, false);
    document.addEventListener('fullscreenVideoAdFail', function (data) {
        hasFullscreenVideoAd = false;
        if (fullscreenVideoLoadFailedCallback) {
            fullscreenVideoLoadFailedCallback(data);
        }
    }, false);
    //Interstitial ad close Listener
    document.addEventListener('fullscreenVideoAdDidClose', function (data) {
        // If we're waiting for a close
        if (isProcessingFullscreenVideo) {
            isProcessingFullscreenVideo = false;
            hasFullscreenVideoAd = false;
            // fire success
            if (fullscreenVideoCompleteCallback) {
                fullscreenVideoCompleteCallback();
            }
            // prepare next interstitial
            if (autoLoadFullscreenVideoAds) {
                prepareFullscreenVideoLoop();
            }
        }
    }, false);

    //interaction callbacks
    document.addEventListener('onRewardVideoCached', function (data) {
        hasRewardedAd = true;
        if (rewardedLoadCallback) {
            rewardedLoadCallback(data);
        }
    }, false);
    document.addEventListener('onRewardVideoError', function (data) {
        switch (data.code) {
        case '20001': // ad failed to load!!!
            hasRewardedAd = false;
            if (rewardedLoadFailedCallback) {
                rewardedLoadFailedCallback(data);
                // prepare next interstitial
                if (autoLoadRewardedAds) {
                    prepareRewardedLoop();
                }
            }
            break;
        }
    }, false);
    //Interstitial ad close Listener
    document.addEventListener('onRewardVideoAdClose', function (data) {
        // if we're waiting for a close event
        if (isProcessingRewarded) {
            isProcessingRewarded = false;
        }
    }, false);
    //Interstitial ad close Listener
    document.addEventListener('onRewardVideoComplete', function (data) {
        // if we're waiting for a close event
        if (isProcessingRewarded) {
            isProcessingRewarded = false;
            if (rewardedCompleteCallback) {
                rewardedCompleteCallback();
            }
        }
    }, false);


    return BytedanceModule;
});